@@ -108,6 +108,8 @@ def sorted_imports(
             comments_above = []
             new_section_output: List[str] = []
             for line in section_output:
+                if not line:
+                    continue
                 if line.startswith("#"):
                     comments_above.append(line)
                 elif comments_above:
@@ -132,7 +134,11 @@ def sorted_imports(
             for line in new_section_output:
                 comments = getattr(line, "comments", ())
                 if comments:
-                    if new_section_output and config.ensure_newline_before_comments:
+                    if (
+                        config.ensure_newline_before_comments
+                        and section_output
+                        and section_output[-1]
+                    ):
                         section_output.append("")
                     section_output.extend(comments)
                 section_output.append(str(line))
@@ -178,16 +184,15 @@ def sorted_imports(
 
     if len(formatted_output) > imports_tail:
         next_construct = ""
-        _in_quote: str = ""
         tail = formatted_output[imports_tail:]
 
         for index, line in enumerate(tail):
-            in_quote = _in_quote
-            should_skip, _in_quote, *_ = parse.skip_line(
+            should_skip, in_quote, *_ = parse.skip_line(
                 line,
-                in_quote=_in_quote,
+                in_quote="",
                 index=len(formatted_output),
-                section_comments=parsed.section_comments,
+                section_comments=config.section_comments,
+                needs_import=False,
             )
             if not should_skip and line.strip():
                 if (
@@ -198,16 +203,9 @@ def sorted_imports(
                     continue
                 next_construct = line
                 break
-            elif not in_quote:
-                parts = line.split()
-                if (
-                    len(parts) >= 3
-                    and parts[1] == "="
-                    and "'" not in parts[0]
-                    and '"' not in parts[0]
-                ):
-                    next_construct = line
-                    break
+            elif in_quote:
+                next_construct = line
+                break
 
         if config.lines_after_imports != -1:
             formatted_output[imports_tail:0] = ["" for line in range(config.lines_after_imports)]
@@ -263,10 +261,10 @@ def _with_from_imports(
         sub_modules = [f"{module}.{from_import}" for from_import in from_imports]
         as_imports = {
             from_import: [
-                f"{from_import} as {as_module}" for as_module in parsed.as_map[sub_module]
+                f"{from_import} as {as_module}" for as_module in parsed.as_map["from"][sub_module]
             ]
             for from_import, sub_module in zip(from_imports, sub_modules)
-            if sub_module in parsed.as_map
+            if sub_module in parsed.as_map["from"]
         }
         if config.combine_as_imports and not ("*" in from_imports and config.combine_star):
             if not config.no_inline_sort:
@@ -275,10 +273,7 @@ def _with_from_imports(
             for from_import in copy.copy(from_imports):
                 if from_import in as_imports:
                     idx = from_imports.index(from_import)
-                    if (
-                        config.keep_direct_and_as_imports
-                        and parsed.imports[section]["from"][module][from_import]
-                    ):
+                    if parsed.imports[section]["from"][module][from_import]:
                         from_imports[(idx + 1) : (idx + 1)] = as_imports.pop(from_import)
                     else:
                         from_imports[idx : (idx + 1)] = as_imports.pop(from_import)
@@ -321,10 +316,7 @@ def _with_from_imports(
                             f"{comments and ';' or config.comment_prefix} " f"{comment}"
                         )
                     if from_import in as_imports:
-                        if (
-                            config.keep_direct_and_as_imports
-                            and parsed.imports[section]["from"][module][from_import]
-                        ):
+                        if parsed.imports[section]["from"][module][from_import]:
                             new_section_output.append(
                                 wrap.line(single_import_line, parsed.line_separator, config)
                             )
@@ -352,26 +344,29 @@ def _with_from_imports(
                     from_comments = parsed.categorized_comments["straight"].get(
                         f"{module}.{from_import}"
                     )
-                    if (
-                        config.keep_direct_and_as_imports
-                        and parsed.imports[section]["from"][module][from_import]
-                    ):
+                    if parsed.imports[section]["from"][module][from_import]:
                         new_section_output.append(
-                            with_comments(
-                                from_comments,
-                                wrap.line(
-                                    import_start + from_import, parsed.line_separator, config
+                            wrap.line(
+                                with_comments(
+                                    from_comments,
+                                    import_start + from_import,
+                                    removed=config.ignore_comments,
+                                    comment_prefix=config.comment_prefix,
                                 ),
-                                removed=config.ignore_comments,
-                                comment_prefix=config.comment_prefix,
+                                parsed.line_separator,
+                                config,
                             )
                         )
                     new_section_output.extend(
-                        with_comments(
-                            from_comments,
-                            wrap.line(import_start + as_import, parsed.line_separator, config),
-                            removed=config.ignore_comments,
-                            comment_prefix=config.comment_prefix,
+                        wrap.line(
+                            with_comments(
+                                from_comments,
+                                import_start + as_import,
+                                removed=config.ignore_comments,
+                                comment_prefix=config.comment_prefix,
+                            ),
+                            parsed.line_separator,
+                            config,
                         )
                         for as_import in as_imports[from_import]
                     )
@@ -391,8 +386,6 @@ def _with_from_imports(
                     comments = None
 
                 for from_import in copy.copy(from_imports):
-                    if from_import in as_imports and not config.keep_direct_and_as_imports:
-                        continue
                     comment = (
                         parsed.categorized_comments["nested"].get(module, {}).pop(from_import, None)
                     )
@@ -416,8 +409,7 @@ def _with_from_imports(
                 while from_imports and (
                     from_imports[0] not in as_imports
                     or (
-                        config.keep_direct_and_as_imports
-                        and config.combine_as_imports
+                        config.combine_as_imports
                         and parsed.imports[section]["from"][module][from_import]
                     )
                 ):
@@ -495,11 +487,12 @@ def _with_straight_imports(
             continue
 
         import_definition = []
-        if module in parsed.as_map:
-            if config.keep_direct_and_as_imports and parsed.imports[section]["straight"][module]:
+        if module in parsed.as_map["straight"]:
+            if parsed.imports[section]["straight"][module]:
                 import_definition.append(f"{import_type} {module}")
             import_definition.extend(
-                f"{import_type} {module} as {as_import}" for as_import in parsed.as_map[module]
+                f"{import_type} {module} as {as_import}"
+                for as_import in parsed.as_map["straight"][module]
             )
         else:
             import_definition.append(f"{import_type} {module}")
